c
c     This subroutine returns the matrix A with the following features:
c
c        (i) a nine-point stencil based on the optimal scheme proposed
c            by Jo, Shin and Suh. 
c            See: Geophysics 61(2) (1996), pp.529--537
c
c       (ii) absorption layer (or "sponge layer") is added and enclosed
c            the original, physical domain. This layer is responsible
c            in reducing nonphysical unwanted reflections at the
c            boundaries. The width of this layer is usually up to 30%
c            of the physical domain to solve.
c
c      (iii) at the end of absorption layer, the 1st order boundary
c            conditions due to Engquist and Majda are imposed. 
c
c       (iv) the corner points are treated in the way proposed by
c            Patrick Joly et. al., which is second order accurate.
c
c
c     Note : this code is somewhat lengthy. Simplification will be made
c            later.
c
      subroutine mka9jabl(xl,yl,ngx,ngy,ngxd,ngyd,wreff,fac,ibc,
     &                    const,a,icase)    
c 
      implicit none
      integer    ix,iy,ngx,ngy,ngxd,ngyd,ngxt,ngyt,icase,ibc(2,2)   
      integer    dxw,nxw,sxw,dxe,nxe,sxe,dyn,nyn,syn,dys,nys,sys 
      real*8     hx,hy,h2,xl,yl,xd,yd 
      real*8     ac,am,as,alf,const,fac,wreff,wvnum,wdata   
      real*8     ac1,ac2,am1,am2,as1,as2,cs,ds 
      complex*16 a(ngx+2*ngxd,ngy+2*ngyd,9),axx,ayy,acx,acy,omg,cidd 
      
      parameter  (cidd = (0.d0,1.d0))
c
c     compute the gridsize; in finite differences, accuracy reason requires
c     that hx = hy. In this case, a O(h^2) scheme is obtained.
c 
c     hx : gridsize in the x-direction
c     hy : gridsize in the y-direction
c     
      hx = xl/dble(float(ngx+1))
      hy = yl/dble(float(ngy+1))      
      h2 = hx**2
c
c     Based on the gridsizes, we determine the sponge layer width.
c     xd : the width of the sponge layer in the x-direction
c     yd : the width of the sponge layer in the y-direction
c
      xd = hx*dble(float(ngxd))
      yd = hy*dble(float(ngyd))
c
c     Total number of gridpoints in all direction
c
      ngxt = ngx + 2*ngxd
      ngyt = ngy + 2*ngyd
c      
c     Coefficients of the stencil (from Jo, Shin and Suh)
c      
      as  = 0.5461d0
      cs  = 0.6248d0
      ds  = 0.9381d-1
      as1 = as
      as2 = ds
      ac1 = (1.d0 - as)/2.d0
      ac2 = (1.d0 - cs - 4.d0*ds)/4.d0
      am1 = -4.d0*as - 4.d0*(1.d0 - as)/2.d0
      am2 = cs
c
c     Interior points
c      
      do iy = 1,ngyt
        if (iy .ge. 1 .and. iy .le. ngyd-1) then
          do ix = 1,ngxt
            if (ix .ge. 1 .and. ix .le. ngxd-1) then
c             (-1,-1)
              wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2 
     &                    + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,1) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0,-1)
              wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix  ))*hx/xd)**2 
     &                    + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,2) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1,-1)
              wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix+1))*hx/xd)**2
     &                    + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,3) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 0)
              wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2
     &                    + const*((ngyd-(iy  ))*hy/yd)**2         
              a(ix,iy,4) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 0)     
              wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix  ))*hx/xd)**2
     &                    + const*((ngyd-(iy  ))*hy/yd)**2         
              a(ix,iy,5) = -1.d0/h2*(am1 + am2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 0)          
              wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix+1))*hx/xd)**2  
     &                    + const*((ngyd-(iy  ))*hy/yd)**2 
              a(ix,iy,6) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 1) 
              wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2
     &                    + const*((ngyd-(iy+1))*hy/yd)**2 
              a(ix,iy,7) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 1)
              wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix  ))*hx/xd)**2
     &                    + const*((ngyd-(iy+1))*hy/yd)**2         
              a(ix,iy,8) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 1)
              wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix+1))*hx/xd)**2
     &                    + const*((ngyd-(iy+1))*hy/yd)**2    
              a(ix,iy,9) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2) 
c
            elseif (ix .eq. ngxd) then
c             (-1,-1)
              wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2 
     &                    + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,1) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0,-1)
              wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,2) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1,-1)
              wvnum = wdata(1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,3) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 0)
              wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2
     &                    + const*((ngyd-(iy  ))*hy/yd)**2         
              a(ix,iy,4) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 0)     
              wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy  ))*hy/yd)**2         
              a(ix,iy,5) = -1.d0/h2*(am1 + am2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 0)          
              wvnum = wdata(1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy  ))*hy/yd)**2 
              a(ix,iy,6) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 1) 
              wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2
     &                    + const*((ngyd-(iy+1))*hy/yd)**2 
              a(ix,iy,7) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 1)
              wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy+1))*hy/yd)**2         
              a(ix,iy,8) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 1)
              wvnum = wdata(1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy+1))*hy/yd)**2    
              a(ix,iy,9) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2) 
c
            elseif (ix .ge. ngxd+1 .and. ix .le. ngxd+ngx) then
c             (-1,-1)
              wvnum = wdata(ix-ngxd-1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,1) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0,-1)
              wvnum = wdata(ix-ngxd  ,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,2) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1,-1)
              wvnum = wdata(ix-ngxd+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,3) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 0)
              wvnum = wdata(ix-ngxd-1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy  ))*hy/yd)**2         
              a(ix,iy,4) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 0)     
              wvnum = wdata(ix-ngxd  ,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy  ))*hy/yd)**2         
              a(ix,iy,5) = -1.d0/h2*(am1 + am2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 0)          
              wvnum = wdata(ix-ngxd+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy  ))*hy/yd)**2 
              a(ix,iy,6) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 1) 
              wvnum = wdata(ix-ngxd-1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy+1))*hy/yd)**2 
              a(ix,iy,7) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 1)
              wvnum = wdata(ix-ngxd  ,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy+1))*hy/yd)**2         
              a(ix,iy,8) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 1)
              wvnum = wdata(ix-ngxd+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy+1))*hy/yd)**2    
              a(ix,iy,9) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2) 
c
            elseif (ix .eq. ngxd+ngx+1) then
c             (-1,-1)
              wvnum = wdata(ngx  ,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,1) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0,-1)
              wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,2) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1,-1)
              wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2
     &                    + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,3) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 0)
              wvnum = wdata(ngx  ,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy  ))*hy/yd)**2         
              a(ix,iy,4) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 0)     
              wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy  ))*hy/yd)**2         
              a(ix,iy,5) = -1.d0/h2*(am1 + am2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 0)          
              wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2  
     &                    + const*((ngyd-(iy  ))*hy/yd)**2 
              a(ix,iy,6) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 1) 
              wvnum = wdata(ngx  ,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy+1))*hy/yd)**2 
              a(ix,iy,7) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 1)
              wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy+1))*hy/yd)**2         
              a(ix,iy,8) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 1)
              wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2
     &                    + const*((ngyd-(iy+1))*hy/yd)**2    
              a(ix,iy,9) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2) 
c
            elseif (ix .ge. ngxd+ngx+2 .and. ix .le. ngxt) then
c             (-1,-1)
              wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)-1)*hx/xd)**2 
     &                    + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,1) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0,-1)
              wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)  )*hx/xd)**2 
     &                    + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,2) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1,-1)
              wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2
     &                    + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,3) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 0)
              wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)-1)*hx/xd)**2
     &                    + const*((ngyd-(iy  ))*hy/yd)**2         
              a(ix,iy,4) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 0)     
              wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)  )*hx/xd)**2
     &                    + const*((ngyd-(iy  ))*hy/yd)**2         
              a(ix,iy,5) = -1.d0/h2*(am1 + am2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 0)          
              wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2  
     &                    + const*((ngyd-(iy  ))*hy/yd)**2 
              a(ix,iy,6) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 1) 
              wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)-1)*hx/xd)**2
     &                    + const*((ngyd-(iy+1))*hy/yd)**2 
              a(ix,iy,7) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 1)
              wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)  )*hx/xd)**2
     &                    + const*((ngyd-(iy+1))*hy/yd)**2         
              a(ix,iy,8) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 1)
              wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2
     &                    + const*((ngyd-(iy+1))*hy/yd)**2    
              a(ix,iy,9) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2) 
c
            endif
          enddo
        elseif (iy .eq. ngyd) then
          do ix = 1,ngxt
            if (ix .ge. 1 .and. ix .le. ngxd-1) then
c             (-1,-1)
              wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2 
     &                    + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,1) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0,-1)
              wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix  ))*hx/xd)**2 
     &                    + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,2) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1,-1)
              wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix+1))*hx/xd)**2
     &                    + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,3) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 0)
              wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2   
              a(ix,iy,4) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 0)     
              wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix  ))*hx/xd)**2
              a(ix,iy,5) = -1.d0/h2*(am1 + am2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 0)          
              wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix+1))*hx/xd)**2  
              a(ix,iy,6) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 1) 
              wvnum = wdata(0,1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2
              a(ix,iy,7) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 1)
              wvnum = wdata(0,1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix  ))*hx/xd)**2
              a(ix,iy,8) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 1)
              wvnum = wdata(0,1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix+1))*hx/xd)**2
              a(ix,iy,9) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2) 
c
            elseif (ix .eq. ngxd) then
c             (-1,-1)
              wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2 
     &                    + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,1) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0,-1)
              wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,2) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1,-1)
              wvnum = wdata(1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,3) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 0)
              wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2     
              a(ix,iy,4) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 0)     
              wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac        
              a(ix,iy,5) = -1.d0/h2*(am1 + am2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 0)          
              wvnum = wdata(1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac
              a(ix,iy,6) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 1) 
              wvnum = wdata(0,1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2
              a(ix,iy,7) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 1)
              wvnum = wdata(0,1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac       
              a(ix,iy,8) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 1)
              wvnum = wdata(1,1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac 
              a(ix,iy,9) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2) 
c
            elseif (ix .ge. ngxd+1 .and. ix .le. ngxd+ngx) then
c             (-1,-1)
              wvnum = wdata(ix-ngxd-1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,1) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0,-1)
              wvnum = wdata(ix-ngxd  ,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,2) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1,-1)
              wvnum = wdata(ix-ngxd+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,3) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 0)
              wvnum = wdata(ix-ngxd-1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac         
              a(ix,iy,4) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 0)     
              wvnum = wdata(ix-ngxd  ,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac          
              a(ix,iy,5) = -1.d0/h2*(am1 + am2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 0)          
              wvnum = wdata(ix-ngxd+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac 
              a(ix,iy,6) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 1) 
              wvnum = wdata(ix-ngxd-1,1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac 
              a(ix,iy,7) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 1)
              wvnum = wdata(ix-ngxd  ,1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac        
              a(ix,iy,8) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 1)
              wvnum = wdata(ix-ngxd+1,1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac   
              a(ix,iy,9) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c
            elseif (ix .eq. ngxd+ngx+1) then
c             (-1,-1)
              wvnum = wdata(ngx  ,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,1) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0,-1)
              wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,2) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1,-1)
              wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2
     &                    + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,3) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 0)
              wvnum = wdata(ngx  ,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac         
              a(ix,iy,4) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 0)     
              wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac         
              a(ix,iy,5) = -1.d0/h2*(am1 + am2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 0)          
              wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2  
              a(ix,iy,6) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 1) 
              wvnum = wdata(ngx  ,1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac 
              a(ix,iy,7) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 1)
              wvnum = wdata(ngx+1,1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac         
              a(ix,iy,8) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 1)
              wvnum = wdata(ngx+1,1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2  
              a(ix,iy,9) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2) 
c
            elseif (ix .ge. ngxd+ngx+2 .and. ix .le. ngxt) then
c             (-1,-1)
              wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)-1)*hx/xd)**2 
     &                    + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,1) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0,-1)
              wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)  )*hx/xd)**2 
     &                    + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,2) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1,-1)
              wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2
     &                    + const*((ngyd-(iy-1))*hy/yd)**2
              a(ix,iy,3) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 0)
              wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)-1)*hx/xd)**2  
              a(ix,iy,4) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 0)     
              wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)  )*hx/xd)**2        
              a(ix,iy,5) = -1.d0/h2*(am1 + am2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 0)          
              wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2  
              a(ix,iy,6) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 1) 
              wvnum = wdata(ngx+1,1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)-1)*hx/xd)**2 
              a(ix,iy,7) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 1)
              wvnum = wdata(ngx+1,1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)  )*hx/xd)**2    
              a(ix,iy,8) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 1)
              wvnum = wdata(ngx+1,1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2
              a(ix,iy,9) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2) 
c
            endif
          enddo
        elseif (iy .ge. ngyd+1 .and. iy .le. ngyd+ngy) then
          do ix = 1,ngxt
            if (ix .ge. 1 .and. ix .le. ngxd-1) then
c             (-1,-1)
              wvnum = wdata(0,iy-ngyd-1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2 
              a(ix,iy,1) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0,-1)
              wvnum = wdata(0,iy-ngyd-1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix  ))*hx/xd)**2 
              a(ix,iy,2) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1,-1)
              wvnum = wdata(0,iy-ngyd-1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix+1))*hx/xd)**2
              a(ix,iy,3) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 0)
              wvnum = wdata(0,iy-ngyd  ,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2         
              a(ix,iy,4) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 0)     
              wvnum = wdata(0,iy-ngyd  ,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix  ))*hx/xd)**2        
              a(ix,iy,5) = -1.d0/h2*(am1 + am2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 0)          
              wvnum = wdata(0,iy-ngyd  ,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix+1))*hx/xd)**2  
              a(ix,iy,6) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 1) 
              wvnum = wdata(0,iy-ngyd+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2 
              a(ix,iy,7) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 1)
              wvnum = wdata(0,iy-ngyd+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix  ))*hx/xd)**2         
              a(ix,iy,8) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 1)
              wvnum = wdata(0,iy-ngyd+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix+1))*hx/xd)**2   
              a(ix,iy,9) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2) 
c
            elseif (ix .eq. ngxd) then
c             (-1,-1)
              wvnum = wdata(0,iy-ngyd-1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2 
              a(ix,iy,1) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0,-1)
              wvnum = wdata(0,iy-ngyd-1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac
              a(ix,iy,2) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1,-1)
              wvnum = wdata(1,iy-ngyd-1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac
              a(ix,iy,3) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 0)
              wvnum = wdata(0,iy-ngyd  ,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2      
              a(ix,iy,4) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 0)     
              wvnum = wdata(0,iy-ngyd  ,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac          
              a(ix,iy,5) = -1.d0/h2*(am1 + am2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 0)          
              wvnum = wdata(1,iy-ngyd  ,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac
              a(ix,iy,6) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 1) 
              wvnum = wdata(0,iy-ngyd+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2
              a(ix,iy,7) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 1)
              wvnum = wdata(0,iy-ngyd+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac         
              a(ix,iy,8) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 1)
              wvnum = wdata(1,iy-ngyd+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac    
              a(ix,iy,9) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2) 
c
            elseif (ix .ge. ngxd+1 .and. ix .le. ngxd+ngx) then
c             (-1,-1)
              wvnum = wdata(ix-ngxd-1,iy-ngyd-1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac
              a(ix,iy,1) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0,-1)
              wvnum = wdata(ix-ngxd  ,iy-ngyd-1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac
              a(ix,iy,2) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1,-1)
              wvnum = wdata(ix-ngxd+1,iy-ngyd-1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac
              a(ix,iy,3) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 0)
              wvnum = wdata(ix-ngxd-1,iy-ngyd  ,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac          
              a(ix,iy,4) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 0)     
              wvnum = wdata(ix-ngxd  ,iy-ngyd  ,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac        
              a(ix,iy,5) = -1.d0/h2*(am1 + am2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 0)          
              wvnum = wdata(ix-ngxd+1,iy-ngyd  ,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac 
              a(ix,iy,6) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 1) 
              wvnum = wdata(ix-ngxd-1,iy-ngyd+1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac  
              a(ix,iy,7) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 1)
              wvnum = wdata(ix-ngxd  ,iy-ngyd+1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac         
              a(ix,iy,8) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 1)
              wvnum = wdata(ix-ngxd+1,iy-ngyd+1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac    
              a(ix,iy,9) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2) 
c
            elseif (ix .eq. ngxd+ngx+1) then
c             (-1,-1)
              wvnum = wdata(ngx  ,iy-ngyd-1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac
              a(ix,iy,1) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0,-1)
              wvnum = wdata(ngx+1,iy-ngyd-1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac
              a(ix,iy,2) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1,-1)
              wvnum = wdata(ngx+1,iy-ngyd-1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2
              a(ix,iy,3) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 0)
              wvnum = wdata(ngx  ,iy-ngyd  ,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac          
              a(ix,iy,4) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 0)     
              wvnum = wdata(ngx+1,iy-ngyd  ,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac          
              a(ix,iy,5) = -1.d0/h2*(am1 + am2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 0)          
              wvnum = wdata(ngx+1,iy-ngyd  ,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2  
              a(ix,iy,6) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 1) 
              wvnum = wdata(ngx  ,iy-ngyd+1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac 
              a(ix,iy,7) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 1)
              wvnum = wdata(ngx+1,iy-ngyd+1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac       
              a(ix,iy,8) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 1)
              wvnum = wdata(ngx+1,iy-ngyd+1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2    
              a(ix,iy,9) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2) 
c
            elseif (ix .ge. ngxd+ngx+2 .and. ix .le. ngxt) then
c             (-1,-1)
              wvnum = wdata(ngx+1,iy-ngyd-1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)-1)*hx/xd)**2 
              a(ix,iy,1) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0,-1)
              wvnum = wdata(ngx+1,iy-ngyd-1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)  )*hx/xd)**2 
              a(ix,iy,2) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1,-1)
              wvnum = wdata(ngx+1,iy-ngyd-1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2
              a(ix,iy,3) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 0)
              wvnum = wdata(ngx+1,iy-ngyd  ,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)-1)*hx/xd)**2         
              a(ix,iy,4) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 0)     
              wvnum = wdata(ngx+1,iy-ngyd  ,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)  )*hx/xd)**2       
              a(ix,iy,5) = -1.d0/h2*(am1 + am2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 0)          
              wvnum = wdata(ngx+1,iy-ngyd  ,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2   
              a(ix,iy,6) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 1) 
              wvnum = wdata(ngx+1,iy-ngyd+1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)-1)*hx/xd)**2
              a(ix,iy,7) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 1)
              wvnum = wdata(ngx+1,iy-ngyd+1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)  )*hx/xd)**2        
              a(ix,iy,8) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 1)
              wvnum = wdata(ngx+1,iy-ngyd+1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2 
              a(ix,iy,9) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2) 
c
            endif
          enddo
        elseif (iy .eq. ngyd+ngy+1) then
          do ix = 1,ngxt
            if (ix .ge. 1 .and. ix .le. ngxd-1) then
c             (-1,-1)
              wvnum = wdata(0,ngy  ,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2 
              a(ix,iy,1) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0,-1)
              wvnum = wdata(0,ngy  ,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix  ))*hx/xd)**2 
              a(ix,iy,2) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1,-1)
              wvnum = wdata(0,ngy  ,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix+1))*hx/xd)**2
              a(ix,iy,3) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 0)
              wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2         
              a(ix,iy,4) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 0)     
              wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix  ))*hx/xd)**2         
              a(ix,iy,5) = -1.d0/h2*(am1 + am2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 0)          
              wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix+1))*hx/xd)**2  
              a(ix,iy,6) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 1) 
              wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2 
              a(ix,iy,7) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 1)
              wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix  ))*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2         
              a(ix,iy,8) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 1)
              wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix+1))*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2    
              a(ix,iy,9) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2) 
c
            elseif (ix .eq. ngxd) then
c             (-1,-1)
              wvnum = wdata(0,ngy  ,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2 
              a(ix,iy,1) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0,-1)
              wvnum = wdata(0,ngy  ,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac 
              a(ix,iy,2) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1,-1)
              wvnum = wdata(1,ngy  ,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac
              a(ix,iy,3) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 0)
              wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2         
              a(ix,iy,4) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 0)     
              wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac        
              a(ix,iy,5) = -1.d0/h2*(am1 + am2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 0)          
              wvnum = wdata(1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac 
              a(ix,iy,6) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 1) 
              wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2 
              a(ix,iy,7) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 1)
              wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2         
              a(ix,iy,8) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 1)
              wvnum = wdata(1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2    
              a(ix,iy,9) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2) 
c
            elseif (ix .ge. ngxd+1 .and. ix .le. ngxd+ngx) then
c             (-1,-1)
              wvnum = wdata(ix-ngxd-1,ngy  ,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac
              a(ix,iy,1) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0,-1)
              wvnum = wdata(ix-ngxd  ,ngy  ,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac
              a(ix,iy,2) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1,-1)
              wvnum = wdata(ix-ngxd+1,ngy  ,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac
              a(ix,iy,3) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 0)
              wvnum = wdata(ix-ngxd-1,ngy+1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac         
              a(ix,iy,4) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 0)     
              wvnum = wdata(ix-ngxd  ,ngy+1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac       
              a(ix,iy,5) = -1.d0/h2*(am1 + am2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 0)          
              wvnum = wdata(ix-ngxd+1,ngy+1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac
              a(ix,iy,6) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 1) 
              wvnum = wdata(ix-ngxd-1,ngy+1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2 
              a(ix,iy,7) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 1)
              wvnum = wdata(ix-ngxd  ,ngy+1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2         
              a(ix,iy,8) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 1)
              wvnum = wdata(ix-ngxd+1,ngy+1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2    
              a(ix,iy,9) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2) 
c
            elseif (ix .eq. ngxd+ngx+1) then
c             (-1,-1)
              wvnum = wdata(ngx  ,ngy  ,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac
              a(ix,iy,1) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0,-1)
              wvnum = wdata(ngx+1,ngy  ,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac 
              a(ix,iy,2) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1,-1)
              wvnum = wdata(ngx+1,ngy  ,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2
              a(ix,iy,3) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 0)
              wvnum = wdata(ngx  ,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac      
              a(ix,iy,4) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 0)     
              wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac       
              a(ix,iy,5) = -1.d0/h2*(am1 + am2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 0)          
              wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2  
              a(ix,iy,6) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 1) 
              wvnum = wdata(ngx  ,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2 
              a(ix,iy,7) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 1)
              wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2         
              a(ix,iy,8) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 1)
              wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2    
              a(ix,iy,9) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2) 
c
            elseif (ix .ge. ngxd+ngx+2 .and. ix .le. ngxt) then
c             (-1,-1)
              wvnum = wdata(ngx+1,ngy  ,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)-1)*hx/xd)**2
              a(ix,iy,1) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0,-1)
              wvnum = wdata(ngx+1,ngy  ,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)  )*hx/xd)**2 
              a(ix,iy,2) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1,-1)
              wvnum = wdata(ngx+1,ngy  ,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2
              a(ix,iy,3) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 0)
              wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)-1)*hx/xd)**2       
              a(ix,iy,4) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 0)     
              wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)  )*hx/xd)**2         
              a(ix,iy,5) = -1.d0/h2*(am1 + am2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 0)          
              wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2  
              a(ix,iy,6) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 1) 
              wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)-1)*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2 
              a(ix,iy,7) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 1)
              wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)  )*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2         
              a(ix,iy,8) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 1)
              wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2    
              a(ix,iy,9) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2) 
c
            endif
          enddo
        elseif (iy .ge. ngyd+ngy+2 .and. iy .le. ngyt) then        
          do ix = 1,ngxt
            if (ix .ge. 1 .and. ix .le. ngxd-1) then
c             (-1,-1)
              wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2 
     &                    + const*((iy-(ngyd+ngy+1)-1)*hy/yd)**2
              a(ix,iy,1) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0,-1)
              wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix  ))*hx/xd)**2 
     &                    + const*((iy-(ngyd+ngy+1)-1)*hy/yd)**2
              a(ix,iy,2) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1,-1)
              wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix+1))*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)-1)*hy/yd)**2
              a(ix,iy,3) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 0)
              wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)  )*hy/yd)**2         
              a(ix,iy,4) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 0)     
              wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix  ))*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)  )*hy/yd)**2         
              a(ix,iy,5) = -1.d0/h2*(am1 + am2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 0)          
              wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix+1))*hx/xd)**2  
     &                    + const*((iy-(ngyd+ngy+1)  )*hy/yd)**2 
              a(ix,iy,6) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 1) 
              wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2 
              a(ix,iy,7) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 1)
              wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix  ))*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2         
              a(ix,iy,8) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 1)
              wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix+1))*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2    
              a(ix,iy,9) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2) 
c
            elseif (ix .eq. ngxd) then
c             (-1,-1)
              wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2 
     &                    + const*((iy-(ngyd+ngy+1)-1)*hy/yd)**2
              a(ix,iy,1) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0,-1)
              wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)-1)*hy/yd)**2
              a(ix,iy,2) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1,-1)
              wvnum = wdata(1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)-1)*hy/yd)**2
              a(ix,iy,3) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 0)
              wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)  )*hy/yd)**2         
              a(ix,iy,4) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 0)     
              wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)  )*hy/yd)**2         
              a(ix,iy,5) = -1.d0/h2*(am1 + am2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 0)          
              wvnum = wdata(1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)  )*hy/yd)**2 
              a(ix,iy,6) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 1) 
              wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ngxd-(ix-1))*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2 
              a(ix,iy,7) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 1)
              wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2         
              a(ix,iy,8) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 1)
              wvnum = wdata(1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2    
              a(ix,iy,9) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2) 
c
            elseif (ix .ge. ngxd+1 .and. ix .le. ngxd+ngx) then
c             (-1,-1)
              wvnum = wdata(ix-ngxd-1,ngy+1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)-1)*hy/yd)**2
              a(ix,iy,1) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0,-1)
              wvnum = wdata(ix-ngxd  ,ngy+1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)-1)*hy/yd)**2
              a(ix,iy,2) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1,-1)
              wvnum = wdata(ix-ngxd+1,ngy+1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)-1)*hy/yd)**2
              a(ix,iy,3) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 0)
              wvnum = wdata(ix-ngxd-1,ngy+1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)  )*hy/yd)**2         
              a(ix,iy,4) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 0)     
              wvnum = wdata(ix-ngxd  ,ngy+1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)  )*hy/yd)**2             
              a(ix,iy,5) = -1.d0/h2*(am1 + am2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 0)          
              wvnum = wdata(ix-ngxd+1,ngy+1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)  )*hy/yd)**2
              a(ix,iy,6) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 1) 
              wvnum = wdata(ix-ngxd-1,ngy+1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2 
              a(ix,iy,7) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 1)
              wvnum = wdata(ix-ngxd  ,ngy+1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2         
              a(ix,iy,8) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 1)
              wvnum = wdata(ix-ngxd+1,ngy+1,ngx,ngy,xl,yl,
     &                      hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2  
              a(ix,iy,9) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2) 
c
            elseif (ix .eq. ngxd+ngx+1) then
c             (-1,-1)
              wvnum = wdata(ngx  ,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)-1)*hy/yd)**2
              a(ix,iy,1) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0,-1)
              wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)-1)*hy/yd)**2
              a(ix,iy,2) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1,-1)
              wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)-1)*hy/yd)**2
              a(ix,iy,3) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 0)
              wvnum = wdata(ngx  ,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)  )*hy/yd)**2         
              a(ix,iy,4) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 0)     
              wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)  )*hy/yd)**2         
              a(ix,iy,5) = -1.d0/h2*(am1 + am2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 0)          
              wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2  
     &                    + const*((iy-(ngyd+ngy+1)  )*hy/yd)**2 
              a(ix,iy,6) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 1) 
              wvnum = wdata(ngx  ,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2 
              a(ix,iy,7) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 1)
              wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2         
              a(ix,iy,8) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 1)
              wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2    
              a(ix,iy,9) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2) 
c
            elseif (ix .ge. ngxd+ngx+2 .and. ix .le. ngxt) then
c             (-1,-1)
              wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)-1)*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)-1)*hy/yd)**2
              a(ix,iy,1) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0,-1)
              wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)  )*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)-1)*hy/yd)**2
              a(ix,iy,2) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1,-1)
              wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)-1)*hy/yd)**2
              a(ix,iy,3) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 0)
              wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)-1)*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)  )*hy/yd)**2       
              a(ix,iy,4) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 0)     
              wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)  )*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)  )*hy/yd)**2         
              a(ix,iy,5) = -1.d0/h2*(am1 + am2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 0)          
              wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2  
     &                    + const*((iy-(ngyd+ngy+1)  )*hy/yd)**2 
              a(ix,iy,6) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             (-1, 1) 
              wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)-1)*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2
              a(ix,iy,7) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 0, 1)
              wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)  )*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2        
              a(ix,iy,8) = -1.d0/h2*(as1 + as2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2)
c             ( 1, 1)
              wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
              alf   = fac + const*((ix-(ngxd+ngx+1)+1)*hx/xd)**2
     &                    + const*((iy-(ngyd+ngy+1)+1)*hy/yd)**2    
              a(ix,iy,9) = -1.d0/h2*(ac1 + ac2*
     &                              (1.d0 - cidd*alf)*h2*wvnum**2) 
c
            endif
          enddo
        endif
      enddo
c
c     Resolve the boundary conditions
c
c     West
      if (ibc(1,1) .eq. 0) then
         dxw = 0.d0
         nxw = 1.d0
         sxw = 0.d0
      elseif (ibc(1,1) .eq. 1) then
         dxw = 1.d0
         nxw = 1.d0
         sxw = 0.d0
      elseif (ibc(1,1) .eq. 2) then
         dxw = 1.d0
         nxw = 1.d0
         sxw = 1.d0
      endif
c     East
      if (ibc(1,2) .eq. 0) then
         dxe = 0.d0
         nxe = 1.d0
         sxe = 0.d0
      elseif (ibc(1,2) .eq. 1) then
         dxe = 1.d0
         nxe = 1.d0
         sxe = 0.d0
      elseif (ibc(1,2) .eq. 2) then
         dxe = 1.d0
         nxe = 1.d0
         sxe = 1.d0
      endif
c     South
      if (ibc(2,1) .eq. 0) then
         dys = 0.d0
         nys = 1.d0
         sys = 0.d0
      elseif (ibc(2,1) .eq. 1) then
         dys = 1.d0
         nys = 1.d0
         sys = 0.d0
      elseif (ibc(2,1) .eq. 2) then
         dys = 1.d0
         nys = 1.d0
         sys = 1.d0
      endif
c     North
      if (ibc(2,2) .eq. 0) then
         dyn = 0.d0
         nyn = 1.d0
         syn = 0.d0
      elseif (ibc(2,2) .eq. 1) then
         dyn = 1.d0
         nyn = 1.d0
         syn = 0.d0
      elseif (ibc(2,2) .eq. 2) then
         dyn = 1.d0
         nyn = 1.d0
         syn = 1.d0
      endif   
c
c     Include the boundary conditions
c
c     N:North    
      do ix = 2,ngxt-1
         if (ix .ge. 2 .and. ix .le. ngxd-1) then
            wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ix,ngyt,4) = a(ix,ngyt,4) 
     &                   + a(ix,ngyt,7)*dyn/(nyn + syn*cidd*omg*hy)
	          a(ix,ngyt,7) = 0.d0
            wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)    
	          omg   = cmplx(1.d0,-fac)*wvnum   
	          a(ix,ngyt,5) = a(ix,ngyt,5) 
     &                   + a(ix,ngyt,8)*dyn/(nyn + syn*cidd*omg*hy)
	          a(ix,ngyt,8) = 0.d0
	          wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ix,ngyt,6) = a(ix,ngyt,6) 
     &                   + a(ix,ngyt,9)*dyn/(nyn + syn*cidd*omg*hy)
            a(ix,ngyt,9) = 0.d0
         elseif (ix .eq. ngxd) then
            wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ix,ngyt,4) = a(ix,ngyt,4) 
     &                   + a(ix,ngyt,7)*dyn/(nyn + syn*cidd*omg*hy)
	          a(ix,ngyt,7) = 0.d0
            wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)    
	          omg   = cmplx(1.d0,-fac)*wvnum   
	          a(ix,ngyt,5) = a(ix,ngyt,5) 
     &                   + a(ix,ngyt,8)*dyn/(nyn + syn*cidd*omg*hy)
	          a(ix,ngyt,8) = 0.d0
	          wvnum = wdata(1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ix,ngyt,6) = a(ix,ngyt,6) 
     &                   + a(ix,ngyt,9)*dyn/(nyn + syn*cidd*omg*hy)
            a(ix,ngyt,9) = 0.d0
         elseif (ix .ge. ngxd+1 .and. ix .le. ngxd+ngx) then
            wvnum = wdata(ix-ngxd-1,ngy+1,ngx,ngy,xl,yl,
     &                    hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ix,ngyt,4) = a(ix,ngyt,4) 
     &                   + a(ix,ngyt,7)*dyn/(nyn + syn*cidd*omg*hy)
	          a(ix,ngyt,7) = 0.d0
            wvnum = wdata(ix-ngxd  ,ngy+1,ngx,ngy,xl,yl,
     &                    hx,hy,wreff,icase)    
	          omg   = cmplx(1.d0,-fac)*wvnum   
	          a(ix,ngyt,5) = a(ix,ngyt,5) 
     &                   + a(ix,ngyt,8)*dyn/(nyn + syn*cidd*omg*hy)
	          a(ix,ngyt,8) = 0.d0
	          wvnum = wdata(ix-ngxd+1,ngy+1,ngx,ngy,xl,yl,
     &                    hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ix,ngyt,6) = a(ix,ngyt,6) 
     &                   + a(ix,ngyt,9)*dyn/(nyn + syn*cidd*omg*hy)
            a(ix,ngyt,9) = 0.d0
         elseif (ix .eq. ngxd+ngx+1) then
            wvnum = wdata(ngx  ,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ix,ngyt,4) = a(ix,ngyt,4) 
     &                   + a(ix,ngyt,7)*dyn/(nyn + syn*cidd*omg*hy)
	          a(ix,ngyt,7) = 0.d0
            wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)    
	          omg   = cmplx(1.d0,-fac)*wvnum   
	          a(ix,ngyt,5) = a(ix,ngyt,5) 
     &                   + a(ix,ngyt,8)*dyn/(nyn + syn*cidd*omg*hy)
	          a(ix,ngyt,8) = 0.d0
	          wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,
     &                          icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ix,ngyt,6) = a(ix,ngyt,6) 
     &                   + a(ix,ngyt,9)*dyn/(nyn + syn*cidd*omg*hy)
            a(ix,ngyt,9) = 0.d0
         elseif (ix .ge. ngxd+ngx+2 .and. ix .le. ngxt-1) then
            wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ix,ngyt,4) = a(ix,ngyt,4) 
     &                   + a(ix,ngyt,7)*dyn/(nyn + syn*cidd*omg*hy)
	          a(ix,ngyt,7) = 0.d0
            wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)    
	          omg   = cmplx(1.d0,-fac)*wvnum   
	          a(ix,ngyt,5) = a(ix,ngyt,5) 
     &                   + a(ix,ngyt,8)*dyn/(nyn + syn*cidd*omg*hy)
	          a(ix,ngyt,8) = 0.d0
	          wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,
     &                          icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ix,ngyt,6) = a(ix,ngyt,6) 
     &                   + a(ix,ngyt,9)*dyn/(nyn + syn*cidd*omg*hy)
            a(ix,ngyt,9) = 0.d0
         endif
      enddo
c
c     S: South
      do ix = 2,ngxt-1
         if (ix .ge. 2 .and. ix .le. ngxd-1) then 
            wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ix,1,4) = a(ix,1,4)
     &                + a(ix,1,1)*dys/(nys+ sys*cidd*omg*hy)
            a(ix,1,1) = 0.d0
	          wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ix,1,5) = a(ix,1,5)
     &                + a(ix,1,2)*dys/(nys+ sys*cidd*omg*hy)
            a(ix,1,2) = 0.d0
	          wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ix,1,6) = a(ix,1,6)
     &                + a(ix,1,3)*dys/(nys+ sys*cidd*omg*hy)
            a(ix,1,3) = 0.d0
         elseif (ix .eq. ngxd) then
            wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ix,1,4) = a(ix,1,4)
     &                + a(ix,1,1)*dys/(nys+ sys*cidd*omg*hy)
            a(ix,1,1) = 0.d0
	          wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ix,1,5) = a(ix,1,5)
     &                + a(ix,1,2)*dys/(nys+ sys*cidd*omg*hy)
            a(ix,1,2) = 0.d0
	          wvnum = wdata(1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ix,1,6) = a(ix,1,6)
     &                + a(ix,1,3)*dys/(nys+ sys*cidd*omg*hy)
            a(ix,1,3) = 0.d0
         elseif (ix .ge. ngxd+1 .and. ix .le. ngxd+ngx) then
            wvnum = wdata(ix-ngxd-1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ix,1,4) = a(ix,1,4)
     &                + a(ix,1,1)*dys/(nys+ sys*cidd*omg*hy)
            a(ix,1,1) = 0.d0
	          wvnum = wdata(ix-ngxd  ,0,ngx,ngy,xl,yl,hx,hy,wreff,
     &                          icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ix,1,5) = a(ix,1,5)
     &                + a(ix,1,2)*dys/(nys+ sys*cidd*omg*hy)
            a(ix,1,2) = 0.d0
	          wvnum = wdata(ix-ngxd+1,0,ngx,ngy,xl,yl,hx,hy,wreff,
     &                          icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ix,1,6) = a(ix,1,6)
     &                + a(ix,1,3)*dys/(nys+ sys*cidd*omg*hy)
            a(ix,1,3) = 0.d0
         elseif (ix .eq. ngxd+ngx+1) then
            wvnum = wdata(ngx  ,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ix,1,4) = a(ix,1,4)
     &                + a(ix,1,1)*dys/(nys+ sys*cidd*omg*hy)
            a(ix,1,1) = 0.d0
	          wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ix,1,5) = a(ix,1,5)
     &                + a(ix,1,2)*dys/(nys+ sys*cidd*omg*hy)
            a(ix,1,2) = 0.d0
	          wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ix,1,6) = a(ix,1,6)
     &                + a(ix,1,3)*dys/(nys+ sys*cidd*omg*hy)
            a(ix,1,3) = 0.d0
         elseif (ix .ge. ngxd+ngx+2 .and. ix .le. ngxt-1) then
            wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ix,1,4) = a(ix,1,4)
     &                + a(ix,1,1)*dys/(nys+ sys*cidd*omg*hy)
            a(ix,1,1) = 0.d0
	          wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ix,1,5) = a(ix,1,5)
     &                + a(ix,1,2)*dys/(nys+ sys*cidd*omg*hy)
            a(ix,1,2) = 0.d0
	          wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ix,1,6) = a(ix,1,6)
     &                + a(ix,1,3)*dys/(nys+ sys*cidd*omg*hy)
            a(ix,1,3) = 0.d0
         endif         
      enddo
c
c     W: West
      do iy = 2,ngyt-1
         if (iy .ge. 2 .and. iy .le. ngyd-1) then
            wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(1,iy,2) = a(1,iy,2)
     &                + a(1,iy,1)*dxw/(nxw+ sxw*cidd*omg*hx)
            a(1,iy,1) = 0.d0
	          wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(1,iy,5) = a(1,iy,5)
     &                + a(1,iy,4)*dxw/(nxw+ sxw*cidd*omg*hx)
            a(1,iy,4) = 0.d0
	          wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(1,iy,8) = a(1,iy,8)
     &                + a(1,iy,7)*dxw/(nxw+ sxw*cidd*omg*hx)
            a(1,iy,7) = 0.d0 
         elseif (iy .eq. ngyd) then
            wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(1,iy,2) = a(1,iy,2)
     &                + a(1,iy,1)*dxw/(nxw+ sxw*cidd*omg*hx)
            a(1,iy,1) = 0.d0
	          wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(1,iy,5) = a(1,iy,5)
     &                + a(1,iy,4)*dxw/(nxw+ sxw*cidd*omg*hx)
            a(1,iy,4) = 0.d0
	          wvnum = wdata(0,1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(1,iy,8) = a(1,iy,8)
     &                + a(1,iy,7)*dxw/(nxw+ sxw*cidd*omg*hx)
            a(1,iy,7) = 0.d0  
         elseif (iy .ge. ngyd+1 .and. iy .le. ngyd+ngy) then
            wvnum = wdata(0,iy-ngyd-1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(1,iy,2) = a(1,iy,2)
     &                + a(1,iy,1)*dxw/(nxw+ sxw*cidd*omg*hx)
            a(1,iy,1) = 0.d0
	          wvnum = wdata(0,iy-ngyd  ,ngx,ngy,xl,yl,hx,hy,wreff,
     &                          icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(1,iy,5) = a(1,iy,5)
     &                + a(1,iy,4)*dxw/(nxw+ sxw*cidd*omg*hx)
            a(1,iy,4) = 0.d0
	          wvnum = wdata(0,iy-ngyd+1,ngx,ngy,xl,yl,hx,hy,wreff,
     &                           icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(1,iy,8) = a(1,iy,8)
     &                + a(1,iy,7)*dxw/(nxw+ sxw*cidd*omg*hx)
            a(1,iy,7) = 0.d0    
         elseif (iy .eq. ngyd+ngy+1) then
            wvnum = wdata(0,ngy  ,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(1,iy,2) = a(1,iy,2)
     &                + a(1,iy,1)*dxw/(nxw+ sxw*cidd*omg*hx)
            a(1,iy,1) = 0.d0
	          wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(1,iy,5) = a(1,iy,5)
     &                + a(1,iy,4)*dxw/(nxw+ sxw*cidd*omg*hx)
            a(1,iy,4) = 0.d0
	          wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(1,iy,8) = a(1,iy,8)
     &                + a(1,iy,7)*dxw/(nxw+ sxw*cidd*omg*hx)
            a(1,iy,7) = 0.d0    
         elseif (iy .ge. ngyd+ngy+2 .and. iy .le. ngyt-1) then
            wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(1,iy,2) = a(1,iy,2)
     &                + a(1,iy,1)*dxw/(nxw+ sxw*cidd*omg*hx)
            a(1,iy,1) = 0.d0
	          wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(1,iy,5) = a(1,iy,5)
     &                + a(1,iy,4)*dxw/(nxw+ sxw*cidd*omg*hx)
            a(1,iy,4) = 0.d0
	          wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(1,iy,8) = a(1,iy,8)
     &                + a(1,iy,7)*dxw/(nxw+ sxw*cidd*omg*hx)
            a(1,iy,7) = 0.d0    
         endif
      enddo
c
c     E: East
      do iy = 2,ngyt-1
         if (iy .ge. 2 .and. iy .le. ngyd-1) then
            wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ngxt,iy,2) = a(ngxt,iy,2)
     &                   + a(ngxt,iy,3)*dxe/(nxe+ sxe*cidd*omg*hx)
            a(ngxt,iy,3) = 0.d0
	          wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ngxt,iy,5) = a(ngxt,iy,5)
     &                   + a(ngxt,iy,6)*dxe/(nxe+ sxe*cidd*omg*hx)
            a(ngxt,iy,6) = 0.d0
	          wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ngxt,iy,8) = a(ngxt,iy,8) 
     &                   + a(ngxt,iy,9)*dxe/(nxe+ sxe*cidd*omg*hx)
            a(ngxt,iy,9) = 0.d0
         elseif (iy .eq. ngyd) then
            wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ngxt,iy,2) = a(ngxt,iy,2)
     &                   + a(ngxt,iy,3)*dxe/(nxe+ sxe*cidd*omg*hx)
            a(ngxt,iy,3) = 0.d0
	          wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ngxt,iy,5) = a(ngxt,iy,5)
     &                   + a(ngxt,iy,6)*dxe/(nxe+ sxe*cidd*omg*hx)
            a(ngxt,iy,6) = 0.d0
	          wvnum = wdata(ngx+1,1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ngxt,iy,8) = a(ngxt,iy,8) 
     &                   + a(ngxt,iy,9)*dxe/(nxe+ sxe*cidd*omg*hx)
            a(ngxt,iy,9) = 0.d0
         elseif (iy .ge. ngyd+1 .and. iy .le. ngyd+ngy) then
            wvnum = wdata(ngx+1,iy-ngyd-1,ngx,ngy,xl,yl,
     &                    hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ngxt,iy,2) = a(ngxt,iy,2)
     &                   + a(ngxt,iy,3)*dxe/(nxe+ sxe*cidd*omg*hx)
            a(ngxt,iy,3) = 0.d0
	          wvnum = wdata(ngx+1,iy-ngyd  ,ngx,ngy,xl,yl,
     &                    hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ngxt,iy,5) = a(ngxt,iy,5)
     &                   + a(ngxt,iy,6)*dxe/(nxe+ sxe*cidd*omg*hx)
            a(ngxt,iy,6) = 0.d0
	          wvnum = wdata(ngx+1,iy-ngyd+1,ngx,ngy,xl,yl,
     &                    hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ngxt,iy,8) = a(ngxt,iy,8) 
     &                   + a(ngxt,iy,9)*dxe/(nxe+ sxe*cidd*omg*hx)
            a(ngxt,iy,9) = 0.d0
         elseif (iy .eq. ngyd+ngy+1) then
            wvnum = wdata(ngx+1,ngy  ,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ngxt,iy,2) = a(ngxt,iy,2)
     &                   + a(ngxt,iy,3)*dxe/(nxe+ sxe*cidd*omg*hx)
            a(ngxt,iy,3) = 0.d0
	          wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,
     &                          icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ngxt,iy,5) = a(ngxt,iy,5)
     &                   + a(ngxt,iy,6)*dxe/(nxe+ sxe*cidd*omg*hx)
            a(ngxt,iy,6) = 0.d0
	          wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,
     &                          icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ngxt,iy,8) = a(ngxt,iy,8) 
     &                   + a(ngxt,iy,9)*dxe/(nxe+ sxe*cidd*omg*hx)
            a(ngxt,iy,9) = 0.d0
         elseif (iy .ge. ngyd+ngy+2 .and. iy .le. ngyt-1) then
            wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ngxt,iy,2) = a(ngxt,iy,2)
     &                   + a(ngxt,iy,3)*dxe/(nxe+ sxe*cidd*omg*hx)
            a(ngxt,iy,3) = 0.d0
	          wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,
     &                          icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ngxt,iy,5) = a(ngxt,iy,5)
     &                   + a(ngxt,iy,6)*dxe/(nxe+ sxe*cidd*omg*hx)
            a(ngxt,iy,6) = 0.d0
	          wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,
     &                          icase)
	          omg   = cmplx(1.d0,-fac)*wvnum
	          a(ngxt,iy,8) = a(ngxt,iy,8) 
     &                   + a(ngxt,iy,9)*dxe/(nxe+ sxe*cidd*omg*hx)
            a(ngxt,iy,9) = 0.d0
         endif
      enddo
c
c     Corner points
c
c     SW : Point(1,1)
      wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
      omg   = cmplx(1.d0,-fac)*wvnum
      axx   = a(1,1,1)*1.d0/(2.d0*(1.d0+cidd*omg*hx))
      ayy   = a(1,1,1)*1.d0/(2.d0*(1.d0+cidd*omg*hy))
      a(1,1,1) = 0.d0
c      
      wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
      omg   = cmplx(1.d0,-fac)*wvnum
      acy   = axx + a(1,1,2)
      a(1,1,5) = a(1,1,5) + acy*dys/(nys + sys*cidd*omg*hy)
      a(1,1,2) = 0.d0
c      
      wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)  
      omg   = cmplx(1.d0,-fac)*wvnum
      acx   = ayy + a(1,1,4)
      a(1,1,5) = a(1,1,5) + acx*dxw/(nxw + sxw*cidd*omg*hx)
      a(1,1,4) = 0.d0
c
      wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
      omg   = cmplx(1.d0,-fac)*wvnum
      a(1,1,6) = a(1,1,6) + a(1,1,3)*dys/(nys+sys*cidd*omg*hy)
      a(1,1,3) = 0.d0
c
      wvnum = wdata(0,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
      omg   = cmplx(1.d0,-fac)*wvnum
      a(1,1,8) = a(1,1,8) + a(1,1,7)*dxw/(nxw+sxw*cidd*omg*hx)
      a(1,1,7) = 0.d0
c
c     SE : Point(ngxt,1)
      wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
      omg   = cmplx(1.d0,-fac)*wvnum
      axx   = a(ngxt,1,3)*1.d0/(2.d0*(1.d0+cidd*omg*hx))
      ayy   = a(ngxt,1,3)*1.d0/(2.d0*(1.d0+cidd*omg*hy))
      a(ngxt,1,3) = 0.d0
c      
      wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
      omg   = cmplx(1.d0,-fac)*wvnum
      acy   = axx + a(ngxt,1,2)
      a(ngxt,1,5) = a(ngxt,1,5) + acy*dys/(nys+sys*cidd*omg*hy)
      a(ngxt,1,2) = 0.d0
c
      wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
      omg   = cmplx(1.d0,-fac)*wvnum
      acx   = ayy + a(ngxt,1,6)
      a(ngxt,1,5) = a(ngxt,1,5) + acx*dxe/(nxe+sxe*cidd*omg*hx)
      a(ngxt,1,6) = 0.d0
c
      wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
      omg   = cmplx(1.d0,-fac)*wvnum
      a(ngxt,1,4) = a(ngxt,1,4) + a(ngxt,1,1)*dys/(nys+sys*cidd*omg*hy)
      a(ngxt,1,1) = 0.d0
c
      wvnum = wdata(ngx+1,0,ngx,ngy,xl,yl,hx,hy,wreff,icase)
      omg   = cmplx(1.d0,-fac)*wvnum
      a(ngxt,1,8) = a(ngxt,1,8) + a(ngxt,1,9)*dxe/(nxe+sxe*cidd*omg*hx)
      a(ngxt,1,9) = 0.d0
c
c     NW: Point(1,ngyt)
      wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
      omg   = cmplx(1.d0,-fac)*wvnum
      axx   = a(1,ngyt,7)*1.d0/(2.d0*(1.d0+cidd*omg*hx))
      ayy   = a(1,ngyt,7)*1.d0/(2.d0*(1.d0+cidd*omg*hy))
      a(1,ngyt,7) = 0.d0
c      
      wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
      omg   = cmplx(1.d0,-fac)*wvnum
      acx   = ayy + a(1,ngyt,4)
      a(1,ngyt,5) = a(1,ngyt,5) + acx*dxw/(nxw+sxw*cidd*omg*hx)
      a(1,ngyt,4) = 0.d0
c      
      wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
      omg   = cmplx(1.d0,-fac)*wvnum
      acy   = axx + a(1,ngyt,8)
      a(1,ngyt,5) = a(1,ngyt,5) + acy*dyn/(nyn+syn*cidd*omg*hy)
      a(1,ngyt,8) = 0.d0
c      
      wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
      omg   = cmplx(1.d0,-fac)*wvnum
      a(1,ngyt,2) = a(1,ngyt,2) + a(1,ngyt,1)*dxw/(nxw+sxw*cidd*omg*hx)
      a(1,ngyt,1) = 0.d0
c      
      wvnum = wdata(0,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
      omg   = cmplx(1.d0,-fac)*wvnum
      a(1,ngyt,6) = a(1,ngyt,6) + a(1,ngyt,9)*dyn/(nyn+syn*cidd*omg*hy)
      a(1,ngyt,9) = 0.d0
c
c     NE: Point(ngxt,ngyt)
      wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
      omg   = cmplx(1.d0,-fac)*wvnum
      axx   = a(ngxt,ngyt,9)*1.d0/(2.d0*(1.d0+cidd*omg*hx))
      ayy   = a(ngxt,ngyt,9)*1.d0/(2.d0*(1.d0+cidd*omg*hy))
      a(ngxt,ngyt,9) = 0.d0
c
      wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
      omg   = cmplx(1.d0,-fac)*wvnum
      acx   = ayy + a(ngxt,ngyt,6)
      a(ngxt,ngyt,5) = a(ngxt,ngyt,5) + acx*dxe/(nxe+sxe*cidd*omg*hx)
      a(ngxt,ngyt,6) = 0.d0
c      
      wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
      omg   = cmplx(1.d0,-fac)*wvnum
      acy   = axx + a(ngxt,ngyt,8)
      a(ngxt,ngyt,5) = a(ngxt,ngyt,5) + acy*dyn/(nyn+syn*cidd*omg*hy)
      a(ngxt,ngyt,8) = 0.d0
c      
      wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
      omg   = cmplx(1.d0,-fac)*wvnum
      a(ngxt,ngyt,2) = a(ngxt,ngyt,2) 
     &               + a(ngxt,ngyt,3)*dxe/(nxe+sxe*cidd*omg*hx)
      a(ngxt,ngyt,3) = 0.d0
c
      wvnum = wdata(ngx+1,ngy+1,ngx,ngy,xl,yl,hx,hy,wreff,icase)
      omg   = cmplx(1.d0,-fac)*wvnum
      a(ngxt,ngyt,4) = a(ngxt,ngyt,4) 
     &               + a(ngxt,ngyt,7)*dyn/(nyn+syn*cidd*omg*hy)
      a(ngxt,ngyt,7) = 0.d0
c
      return
      end

